﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace Magify
{
    internal abstract class Section
    {
        public abstract event Action OnChanged;
        public abstract bool HasData { get; }
        public abstract string Name { get; }

        public abstract void WriteTo(BinaryWriter writer);
        public abstract void ReadFrom(BinaryReader reader);
        public abstract void Reset();
    }

    internal abstract class Section<TType, TData> : Section
    {
        public override bool HasData => Data.Count > 0;
        public BinaryTypeHandler<TType> BinaryType { get; set; }
        protected Dictionary<string, TData> Data { get; } = new();

        protected Section(BinaryTypeHandler<TType> binaryType)
        {
            BinaryType = binaryType;
        }

        public abstract bool Remove(string key);

        public int Remove(Func<string, bool> predicate)
        {
            var keys = Data.Keys.Where(predicate).ToList();
            foreach (var key in keys)
            {
                Remove(key);
            }
            return keys.Count;
        }

        protected (bool Exists, TData Value) TryGet(string key)
        {
            return Data.TryGetValue(key, out var value) ? (true, value) : (false, default);
        }
    }
}