﻿using FluentAssertions;
using Magify.Model;
using NUnit.Framework;

namespace Magify.Tests
{
    internal partial class MagifyClientTests
    {
        public class IsGeoIpEnabled : MagifyClientTestsPart
        {
            private static object[] DeviceToIsGeoIpRequirement => new object[]
            {
                new object[] {EditorDevice.GetDefaultAndroidDevice(), true},
                new object[] {EditorDevice.GetDefaultIOSDevice(), true},
            };

            [Test]
            [TestCaseSource(nameof(DeviceToIsGeoIpRequirement))]
            public void WhenMagifyClientInit_AuthTokenPayloadMustNotContainsIsIdfaAvailable(EditorDevice device, bool requiresIsGeoIpFlag)
            {
                // Arrange
                using var magifyClient = CreateWithDeviceAndInit(device);

                // Act
                magifyClient.IsGeoIpEnabled.Value = false;

                // Assert
                var tokenPayload = (AuthTokenPayload)GetServerApi(magifyClient).CreateAuthTokenRequestMessage().Payload;
                if (requiresIsGeoIpFlag) tokenPayload.IsGeoIPEnabled.Should().HaveValue();
                else tokenPayload.IsGeoIPEnabled.Should().NotHaveValue();
            }
        }
    }
}